"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateBinCommand = void 0;
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var ora_1 = tslib_1.__importDefault(require("ora"));
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var enquirer_1 = require("enquirer");
var errors_1 = require("../../errors");
var package_utils_1 = require("../../helpers/utils/package.utils");
var color_utils_1 = require("../../helpers/utils/color.utils");
var setup_1 = require("../../setup");
var utils_1 = require("../../setup/package/utils");
var package_const_1 = require("../../helpers/constants/package.const");
var package_messages_1 = require("../../helpers/messages/package.messages");
var templateOptions = Object.keys(setup_1.packageTemplates);
var featureOptions = Object.keys(setup_1.featureTemplates);
exports.generateBinCommand = function (prog) {
    prog
        .command('generate <pkg>', 'Generate a new package.', {
        // eslint-disable no-param-reassign
        // eslint-disable-next-line @typescript-eslint/ban-ts-ignore
        // @ts-ignore
        alias: ['g']
    })
        .example('generate packageName')
        .option('t, template', "Specify a template.\n     Available templates: [" + templateOptions.join(', ') + "]\n     \n     ")
        .example("generate packageName --template " + templateOptions[0])
        .option('f, feature', "Specify a feature.\n     Available features: [" + featureOptions.join(', ') + "]\n     \n     ")
        .example("generate packageName --feature " + featureOptions[0])
        .action(function (packageName, opts) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        function getProjectPath(projectPath) {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var isExist, prompt, nextPackageName, nextProjectPath;
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, fs_extra_1.default.pathExists(projectPath)];
                        case 1:
                            isExist = _a.sent();
                            if (!isExist) {
                                return [2 /*return*/, projectPath];
                            }
                            console.log(projectPath);
                            bootSpinner.fail(failed());
                            prompt = new enquirer_1.Input({
                                message: exists(),
                                initial: copy(),
                                result: function (v) { return v.trim(); }
                            });
                            return [4 /*yield*/, prompt.run()];
                        case 2:
                            nextPackageName = _a.sent();
                            return [4 /*yield*/, fs_extra_1.default.realpath(process.cwd())];
                        case 3:
                            nextProjectPath = (_a.sent()) + "/" + cliConfig.workspaces + nextPackageName;
                            return [2 /*return*/, getProjectPath(nextProjectPath)];
                    }
                });
            });
        }
        var _a, wrongWorkspace, successfulConfigure, failedConfigure, successful, copy, failed, script, generating, exists, invalidTemplate, preparedPackage, preparingPackage, cliConfig, currentPath, packageJsonPath, _b, name_1, workspaces, license, isPackagePrivate, workspacePackages, hasWorkspace, err_1, bootSpinner, realPath, projectPath, prompt_1, _c, _d, author, licenseInput, templateConfig, generatePackageJson, pkgJson, err_2, dependencies, installSpinner, _e, _f, err_3;
        return tslib_1.__generator(this, function (_g) {
            switch (_g.label) {
                case 0:
                    _a = new package_messages_1.PackageMessages(packageName), wrongWorkspace = _a.wrongWorkspace, successfulConfigure = _a.successfulConfigure, failedConfigure = _a.failedConfigure, successful = _a.successful, copy = _a.copy, failed = _a.failed, script = _a.script, generating = _a.generating, exists = _a.exists, invalidTemplate = _a.invalidTemplate, preparedPackage = _a.preparedPackage, preparingPackage = _a.preparingPackage;
                    cliConfig = {
                        template: null,
                        workspaces: null,
                        rootWorkspaceName: null,
                        license: null
                    };
                    _g.label = 1;
                case 1:
                    _g.trys.push([1, 4, , 5]);
                    return [4 /*yield*/, fs_extra_1.default.realpath(process.cwd())];
                case 2:
                    currentPath = _g.sent();
                    packageJsonPath = path_1.default.resolve(currentPath, package_const_1.PACKAGE_JSON);
                    return [4 /*yield*/, fs_extra_1.default.readJSON(packageJsonPath)];
                case 3:
                    _b = (_g.sent()), name_1 = _b.name, workspaces = _b.workspaces, license = _b.license, isPackagePrivate = _b.private;
                    workspacePackages = package_utils_1.findWorkspacePackages(workspaces);
                    hasWorkspace = workspacePackages.length > 0;
                    if (!hasWorkspace || !isPackagePrivate) {
                        throw new errors_1.WrongWorkspaceError(wrongWorkspace());
                    }
                    cliConfig.license = license;
                    cliConfig.rootWorkspaceName = name_1;
                    cliConfig.workspaces = package_utils_1.findPackageSetupPath(workspacePackages);
                    return [3 /*break*/, 5];
                case 4:
                    err_1 = _g.sent();
                    if (err_1.isWrongWorkspace) {
                        console.log(color_utils_1.error(err_1));
                    }
                    else {
                        console.log(color_utils_1.error(new errors_1.NoPackageJsonError(script())));
                        errors_1.logError(err_1);
                    }
                    process.exit(1);
                    return [3 /*break*/, 5];
                case 5:
                    bootSpinner = ora_1.default(generating());
                    _g.label = 6;
                case 6:
                    _g.trys.push([6, 18, , 19]);
                    return [4 /*yield*/, fs_extra_1.default.realpath(process.cwd())];
                case 7:
                    realPath = _g.sent();
                    return [4 /*yield*/, getProjectPath(realPath + "/" + cliConfig.workspaces + "/" + packageName)];
                case 8:
                    projectPath = _g.sent();
                    prompt_1 = new enquirer_1.Select({
                        message: 'Choose a template',
                        choices: templateOptions.map(function (option) { return ({
                            name: option,
                            message: color_utils_1.info(option)
                        }); })
                    });
                    if (!opts.template) return [3 /*break*/, 11];
                    cliConfig.template = opts.template.trim();
                    if (!!prompt_1.choices.find(function (choice) { return choice.name === cliConfig.template; })) return [3 /*break*/, 10];
                    bootSpinner.fail(invalidTemplate(cliConfig.template));
                    _c = cliConfig;
                    return [4 /*yield*/, prompt_1.run()];
                case 9:
                    _c.template = _g.sent();
                    _g.label = 10;
                case 10: return [3 /*break*/, 13];
                case 11:
                    _d = cliConfig;
                    return [4 /*yield*/, prompt_1.run()];
                case 12:
                    _d.template = _g.sent();
                    _g.label = 13;
                case 13:
                    bootSpinner.start();
                    return [4 /*yield*/, fs_extra_1.default.copy(path_1.default.resolve(__dirname, "../../../../templates/package/" + cliConfig.template), projectPath, {
                            overwrite: true
                        })];
                case 14:
                    _g.sent();
                    author = package_utils_1.getAuthorName();
                    if (!!author) return [3 /*break*/, 16];
                    bootSpinner.stop();
                    licenseInput = new enquirer_1.Input({
                        name: 'author',
                        message: 'Who is the package author?'
                    });
                    return [4 /*yield*/, licenseInput.run()];
                case 15:
                    author = _g.sent();
                    package_utils_1.setAuthorName(author);
                    bootSpinner.start();
                    _g.label = 16;
                case 16:
                    process.chdir(projectPath);
                    templateConfig = setup_1.packageTemplates[cliConfig.template];
                    generatePackageJson = utils_1.composePackageJson(templateConfig);
                    pkgJson = generatePackageJson({
                        author: author,
                        name: packageName,
                        rootName: cliConfig.rootWorkspaceName,
                        license: cliConfig.license
                    });
                    return [4 /*yield*/, fs_extra_1.default.outputJSON(path_1.default.resolve(projectPath, package_const_1.PACKAGE_JSON), pkgJson)];
                case 17:
                    _g.sent();
                    bootSpinner.succeed(successful());
                    return [3 /*break*/, 19];
                case 18:
                    err_2 = _g.sent();
                    bootSpinner.fail(failed());
                    errors_1.logError(err_2);
                    process.exit(1);
                    return [3 /*break*/, 19];
                case 19:
                    dependencies = setup_1.packageTemplates[cliConfig.template].dependencies;
                    installSpinner = ora_1.default(preparingPackage(dependencies.sort())).start();
                    _g.label = 20;
                case 20:
                    _g.trys.push([20, 25, , 26]);
                    return [4 /*yield*/, package_utils_1.sortPackageJson()];
                case 21:
                    _g.sent();
                    return [4 /*yield*/, package_utils_1.prettifyPackageJson()];
                case 22:
                    _g.sent();
                    return [4 /*yield*/, package_utils_1.buildPackage()];
                case 23:
                    _g.sent();
                    installSpinner.succeed(successfulConfigure());
                    _f = (_e = console).log;
                    return [4 /*yield*/, preparedPackage(packageName)];
                case 24:
                    _f.apply(_e, [_g.sent()]);
                    return [3 /*break*/, 26];
                case 25:
                    err_3 = _g.sent();
                    installSpinner.fail(failedConfigure());
                    errors_1.logError(err_3);
                    process.exit(1);
                    return [3 /*break*/, 26];
                case 26: return [2 /*return*/];
            }
        });
    }); });
};
