"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.installBinCommand = void 0;
var tslib_1 = require("tslib");
var execa_1 = tslib_1.__importDefault(require("execa"));
var ora_1 = tslib_1.__importDefault(require("ora"));
var errors_1 = require("../../errors");
var dependency_utils_1 = require("../../helpers/utils/dependency.utils");
var install_messages_1 = require("../../helpers/messages/install.messages");
var package_utils_1 = require("../../helpers/utils/package.utils");
exports.installBinCommand = function (prog) {
    prog
        .command('install', "Install one or more dependencies to the workspace root. Run this script inside any package and re-space will add peer dependencies as well.", {
        // eslint-disable no-param-reassign
        // eslint-disable-next-line @typescript-eslint/ban-ts-ignore
        // @ts-ignore
        alias: ['i']
    })
        .example('install libraryName')
        .option('d, dev', 'Install development dependencies')
        .example("install libraryName --dev")
        .action(function (opts) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var dependencies, dev, d, dependencyFlag, _a, installing, failed, successful, bootSpinner, workspacePackage, workspaceRoot, e_1;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    dependencies = opts._, dev = opts.dev, d = opts.d;
                    dependencyFlag = dependency_utils_1.defineDependencyFlag(dev, d);
                    _a = new install_messages_1.InstallMessages(dependencies), installing = _a.installing, failed = _a.failed, successful = _a.successful;
                    bootSpinner = ora_1.default(installing());
                    if (typeof dev === 'string' || typeof d === 'string') {
                        dependencies.push(dev);
                    }
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, 8, , 9]);
                    bootSpinner.start();
                    return [4 /*yield*/, package_utils_1.findWorkspacePackage()];
                case 2:
                    workspacePackage = _b.sent();
                    return [4 /*yield*/, package_utils_1.findWorkspaceRoot()];
                case 3:
                    workspaceRoot = _b.sent();
                    if (!(workspacePackage !== null)) return [3 /*break*/, 5];
                    return [4 /*yield*/, execa_1.default("yarn add " + dependencies.join(' '), ['--peer'], {
                            cwd: workspacePackage
                        })];
                case 4:
                    _b.sent();
                    _b.label = 5;
                case 5:
                    if (!(workspaceRoot !== null)) return [3 /*break*/, 7];
                    return [4 /*yield*/, execa_1.default("yarn add " + dependencies.join(' ') + " " + dependencyFlag, ['--exact', '-W'], { cwd: workspaceRoot })];
                case 6:
                    _b.sent();
                    _b.label = 7;
                case 7:
                    bootSpinner.succeed(successful());
                    return [3 /*break*/, 9];
                case 8:
                    e_1 = _b.sent();
                    bootSpinner.fail(failed());
                    errors_1.logError(e_1);
                    return [3 /*break*/, 9];
                case 9: return [2 /*return*/];
            }
        });
    }); });
};
