"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PackageMessages = void 0;
var tslib_1 = require("tslib");
var color_utils_1 = require("../utils/color.utils");
var package_const_1 = require("../constants/package.const");
var PackageMessages = /** @class */ (function () {
    // eslint-disable-next-line no-empty-function
    function PackageMessages(packageName) {
        var _this = this;
        this.packageName = packageName;
        this.script = function () { return color_utils_1.inverse(" generate " + _this.packageName + " "); };
        this.wrongWorkspace = function () { return "\n    Make sure you run the script " + _this.script() + " from the workspace root\n\n    The workspace root " + package_const_1.PACKAGE_JSON + " should have:\n        private: false;\n        workspaces: ['packages/*']\n          "; };
        this.copy = function () { return _this.packageName + "-copy"; };
        this.successful = function () { return "Generated " + color_utils_1.success(_this.packageName) + " package"; };
        this.generating = function () { return "Generating " + color_utils_1.info(_this.packageName) + " package..."; };
        this.failed = function () { return "Failed to generate " + color_utils_1.error(_this.packageName) + " package"; };
        this.successfulConfigure = function () { return 'The package successfully configured'; };
        this.failedConfigure = function () { return "Failed to fully configure the package"; };
        this.invalidTemplate = function (template) { return "Invalid template " + color_utils_1.error(template); };
        this.exists = function () {
            return "A folder named " + color_utils_1.error(_this.packageName) + " already exists! " + color_utils_1.bold('Choose a different name');
        };
        this.preparingPackage = function (packages) {
            var pkgText = packages.map(function (pkg) { return "     " + color_utils_1.info(pkg); }).join('\n');
            return packages.length > 0
                ? "Preparing a package with the following peer dependencies: \n" + pkgText + "\n"
                : 'Preparing the package...';
        };
        this.preparedPackage = function (projectName) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var commands;
            return tslib_1.__generator(this, function (_a) {
                commands = {
                    install: 'yarn install',
                    start: 'yarn start',
                    build: 'yarn build',
                    test: 'yarn test'
                };
                return [2 /*return*/, "\n  " + color_utils_1.success('Awesome!') + " You're now ready to start coding.\n  \n  There is no need to run " + color_utils_1.info(commands.install) + ", since all peer dependencies are already in the workspace root\n  \n  So your next steps are:\n    " + color_utils_1.info("cd packages/" + projectName) + "\n  \n  To start developing (rebuilds the bundle on changes):\n    " + color_utils_1.info(commands.start) + "\n  \n  To build the bundle:\n    " + color_utils_1.info(commands.build) + "\n    \n  To test the package:\n    " + color_utils_1.info(commands.test) + "\n    \n  " + color_utils_1.highlight('Happy coding :)')];
            });
        }); };
    }
    return PackageMessages;
}());
exports.PackageMessages = PackageMessages;
