"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findWorkspacePackage = exports.findWorkspaceRoot = exports.findPackageSetupPath = exports.findWorkspacePackages = exports.buildPackage = exports.prettifyPackageJson = exports.sortPackageJson = exports.setAuthorName = exports.getAuthorName = exports.safePackageName = void 0;
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var shelljs_1 = tslib_1.__importDefault(require("shelljs"));
var execa_1 = tslib_1.__importDefault(require("execa"));
var package_const_1 = require("../constants/package.const");
var errors_1 = require("../../errors");
exports.safePackageName = function (name) {
    return name
        .toLowerCase()
        .replace(/(^@.*\/)|((^[^a-zA-Z]+)|[^\w.-])|([^a-zA-Z0-9]+$)/g, '');
};
function getAuthorName() {
    var author = '';
    author = shelljs_1.default
        .exec('npm config get init-author-name', { silent: true })
        .stdout.trim();
    if (author) {
        return author;
    }
    author = shelljs_1.default.exec('git config user.name', { silent: true }).stdout.trim();
    if (author) {
        setAuthorName(author);
        return author;
    }
    author = shelljs_1.default
        .exec('npm config get init-author-email', { silent: true })
        .stdout.trim();
    if (author) {
        return author;
    }
    author = shelljs_1.default.exec('git config user.email', { silent: true }).stdout.trim();
    if (author) {
        return author;
    }
    return author;
}
exports.getAuthorName = getAuthorName;
function setAuthorName(author) {
    shelljs_1.default.exec("npm config set init-author-name \"" + author + "\"", { silent: true });
}
exports.setAuthorName = setAuthorName;
exports.sortPackageJson = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () { return tslib_1.__generator(this, function (_a) {
    return [2 /*return*/, execa_1.default('sort-package-json')];
}); }); };
exports.prettifyPackageJson = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () { return tslib_1.__generator(this, function (_a) {
    return [2 /*return*/, execa_1.default("prettier --write " + package_const_1.PACKAGE_JSON)];
}); }); };
exports.buildPackage = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () { return tslib_1.__generator(this, function (_a) {
    return [2 /*return*/, execa_1.default('yarn build')];
}); }); };
exports.findWorkspacePackages = function (workspaces) {
    if (workspaces === undefined) {
        return [];
    }
    if ('packages' in workspaces) {
        return workspaces.packages || [];
    }
    return workspaces;
};
exports.findPackageSetupPath = function (packages) {
    return packages.reduce(function (res, pkg) {
        return pkg[pkg.length - 1] === '*' ? pkg.slice(0, pkg.length - 1) : res;
    }, '/');
};
function findWorkspacePath(possiblePath, conditionCallback) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var packageJsonPath, _a, _b, err_1;
        return tslib_1.__generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    _c.trys.push([0, 3, , 4]);
                    // TODO: check is this condition ok
                    if (possiblePath.length < 10) {
                        throw new Error();
                    }
                    packageJsonPath = path_1.default.resolve(possiblePath, 'package.json');
                    _a = fs_extra_1.default.existsSync(packageJsonPath);
                    if (!_a) return [3 /*break*/, 2];
                    _b = conditionCallback;
                    return [4 /*yield*/, fs_extra_1.default.readJSON(packageJsonPath)];
                case 1:
                    _a = _b.apply(void 0, [(_c.sent())]);
                    _c.label = 2;
                case 2: return [2 /*return*/, _a ? possiblePath
                        : findWorkspacePath(path_1.default.resolve(possiblePath, '..'), conditionCallback)];
                case 3:
                    err_1 = _c.sent();
                    errors_1.logError(err_1);
                    return [2 /*return*/, null];
                case 4: return [2 /*return*/];
            }
        });
    });
}
exports.findWorkspaceRoot = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var _a;
    return tslib_1.__generator(this, function (_b) {
        switch (_b.label) {
            case 0:
                _a = findWorkspacePath;
                return [4 /*yield*/, fs_extra_1.default.realpath(process.cwd())];
            case 1: return [2 /*return*/, _a.apply(void 0, [_b.sent(),
                    function (pkg) { return pkg.workspaces !== undefined && pkg.private; }])];
        }
    });
}); };
exports.findWorkspacePackage = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var _a;
    return tslib_1.__generator(this, function (_b) {
        switch (_b.label) {
            case 0:
                _a = findWorkspacePath;
                return [4 /*yield*/, fs_extra_1.default.realpath(process.cwd())];
            case 1: return [2 /*return*/, _a.apply(void 0, [_b.sent(),
                    function (pkg) { return pkg.workspace && !pkg.private; }])];
        }
    });
}); };
